<?php

namespace Drupal\commerce_stripe_payment_request_button\Controller;

use Drupal;
use Drupal\commerce_price\Price;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_stripe_payment_request_button\Services\PaymentRequestButtonHelperServiceInterface;

class PaymentRequestButtonShipping {
  /**
   * The Payment Request button service.
   *
   * @var \Drupal\commerce_stripe_payment_request_button\Services\PaymentRequestButtonHelperServiceInterface
   */
  protected $paymentRequestButtonService;


  public function __construct() {
    $this->paymentRequestButtonService = Drupal::service('commerce_stripe_payment_request_button.helper');
  }

  public function handleShipping(EntityInterface $commerce_order, Request $request) {
    // Transform JSON values to array
    $values = Json::decode($request->getContent());
    $order = $commerce_order;
    $responseShippingMethods = [];
    if ($order->hasItems()) {
      //if country not blocked
      if ($this->paymentRequestButtonService->checkShippingAvailabilityByCountry($order, $values['shippingAddress'])) {
        $shipment = $this->paymentRequestButtonService->processShipment($order);
        /*
        upon changing the shipping address, We need to set the profiles
        so We can calculate the shipping rates and/or taxes
        */

        // Find rate, we use the first one, pretending there is only one we want to use, fixed rate for order
        $shipping_method_storage = Drupal::entityTypeManager()
          ->getStorage('commerce_shipping_method');
        $shipping_methods = $shipping_method_storage->loadMultipleForShipment($shipment);

        // Loop through shipping methods and add them to array so it can be sent as Response to Stripe
        foreach ($shipping_methods as $shipping_method) {
          $shipment->setShippingMethod($shipping_method);
          $shipping_method_plugin = $shipping_method->getPlugin();
          $shipping_rates = $shipping_method_plugin->calculateRates($shipment);
          $shipping_rate = reset($shipping_rates);
          $shipping_method_amount = $shipping_rate->getAmount();

          //if the order currency is different from the default currency, check the if there is an correct shipping field set
          if (
            $shipping_method_amount->getCurrencyCode() != $order->getTotalPrice()
            ->getCurrencyCode()
            && isset($shipping_method_plugin->getConfiguration()['fields'][$order->getTotalPrice()
              ->getCurrencyCode()])
          ) {
            $newAmount = $shipping_method_plugin->getConfiguration()['fields'][$order->getTotalPrice()
              ->getCurrencyCode()];
            if (is_numeric($newAmount['number']) && $newAmount['number'] > 0) {
              $shipping_method_amount = new Price($newAmount['number'], $newAmount['currency_code']);
            }
          }
          //amount is not zero and there are not currency fields appended We use the commerce_currency_resolver to convert the amount
          elseif (
            $shipping_method_amount->getNumber() > 0 &&
            $shipping_method_amount->getCurrencyCode() != $order->getTotalPrice()
            ->getCurrencyCode()
            && isset($shipping_method_plugin->getConfiguration()['fields'][$order->getTotalPrice()
              ->getCurrencyCode()])
          ) {
            $shipping_method_amount = Drupal::service('commerce_currency_resolver.calculator')
              ->priceConversion($shipping_method_amount, $order->getTotalPrice()
                ->getCurrencyCode());
          }

          $shipment->setAmount($shipping_method_amount);
          $responseShippingMethods[] = [
            'id' => $shipping_method->id(),
            'label' => $shipping_method->getName(),
            'detail' => $shipping_rate->getService()->getLabel(),
            'amount' => $this->paymentRequestButtonService->toMinorUnits($shipping_method_amount),
          ];
        }
      }
    }


    $response['shippingOptions'] = $responseShippingMethods;
    return new JsonResponse($response);
  }
}
